//
//  PhysicsScene.m
//  SpriteKitDemo2
//

#import "PhysicsScene.h"
#import "MyScene.h"

//カテゴリビットマスク
static const uint32_t missileCategory	=  0x1 << 0;	//ミサイル
static const uint32_t rockCategory		=  0x1 << 1;	//隕石
static const uint32_t earthCategory		=  0x1 << 2;	//地球

@implementation PhysicsScene{
	
	SKTexture	*_textureMissile;
	SKTexture	*_textureRock;
	
	SKEmitterNode	*_particleFire;
	SKEmitterNode	*_particleSpark;
	
	BOOL			_gameOver;
}


//シーンがビューに表示されたときに呼ばれる
- (void)didMoveToView:(SKView *)view
{
    self.backgroundColor = [SKColor blackColor];
    self.scaleMode = SKSceneScaleModeAspectFit;
	
	_textureMissile = [SKTexture textureWithImageNamed:@"Missile"];
	_textureRock = [SKTexture textureWithImageNamed:@"Rock"];
	
	//地球
	SKSpriteNode* earth = [SKSpriteNode spriteNodeWithImageNamed:@"Earth"];
	earth.position = CGPointMake(self.size.width/2, earth.size.height/2);
	earth.name = @"earth";
	earth.physicsBody = [SKPhysicsBody bodyWithRectangleOfSize:earth.size];
	earth.physicsBody.dynamic = NO;
	[self addChild:earth];
	//接触設定
	earth.physicsBody.categoryBitMask = earthCategory;		//カテゴリー（地球）
	earth.physicsBody.contactTestBitMask = rockCategory;	//ヒットテストするオブジェクト（隕石）
	earth.physicsBody.collisionBitMask = rockCategory;		//接触できるオブジェクト（隕石）
	
	
	//無重力にする
	self.physicsWorld.gravity = CGVectorMake(0, 0);
	
	//接触デリゲート
	self.physicsWorld.contactDelegate = self;
	
	
	//隕石を一定間隔でランダムに作る
	SKAction *makeRocks = [SKAction sequence: @[
												[SKAction performSelector:@selector(addRock) onTarget:self],
												[SKAction waitForDuration:0.8 withRange:1.6]]];
	[self runAction: [SKAction repeatActionForever:makeRocks]];
}

//隕石作成
- (void) addRock
{
	if(_gameOver){
		return;
	}
	
    SKSpriteNode *rock = [SKSpriteNode spriteNodeWithTexture:_textureRock];
	rock.position = CGPointMake(skRand(20, self.size.width-40), self.size.height+50);
	rock.name = @"rock";
	rock.physicsBody = [SKPhysicsBody bodyWithRectangleOfSize:rock.size];
	[self addChild:rock];
	
	//接触設定
	rock.physicsBody.categoryBitMask = rockCategory;		//カテゴリー（隕石）
	rock.physicsBody.contactTestBitMask = missileCategory;	//ヒットテストするオブジェクト（ミサイル）
	rock.physicsBody.collisionBitMask = missileCategory;	//接触できるオブジェクト（ミサイル）
	
	//重力の影響を受けない
	rock.physicsBody.affectedByGravity = NO;
	//下方向に回転させて発射
	rock.physicsBody.velocity = CGVectorMake(0, -(70+skRand(0, 20)));
	[rock.physicsBody applyTorque:0.02];
}

static inline CGFloat skRand(CGFloat low, CGFloat high) {
	
	return skRandf() * (high - low) + low;
}
static inline CGFloat skRandf() {
	
	return rand() / (CGFloat) RAND_MAX;
}



//削除
-(void)didSimulatePhysics
{
	[self enumerateChildNodesWithName:@"rock" usingBlock:^(SKNode *node, BOOL *stop) {
        if (node.position.y < 0 || node.position.x < 0 || node.position.x > 320)
            [node removeFromParent];
	}];
	[self enumerateChildNodesWithName:@"Missile" usingBlock:^(SKNode *node, BOOL *stop) {
        if (node.position.y > self.frame.size.height || node.position.y < 0 || node.position.x < 0 || node.position.x > 320)
            [node removeFromParent];
	}];
	
}

-(void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event
{
	if(_gameOver){
		// シーンを推移させる
		MyScene* newScene  = [[MyScene alloc] initWithSize:self.size];
		
		SKTransition *transition = [SKTransition fadeWithDuration:1.2];
		
		[self.view presentScene:newScene transition:transition];
		return;
	}
	
    for (UITouch *touch in touches) {
		
		CGPoint location = [touch locationInNode:self];
		
		//ミサイル作成
		SKSpriteNode *missaile = [SKSpriteNode spriteNodeWithTexture:_textureMissile];
        missaile.position = CGPointMake(location.x, 0);
		missaile.name = @"Missile";
        [self addChild:missaile];
		missaile.physicsBody = [SKPhysicsBody bodyWithRectangleOfSize:missaile.size];
		//接触設定
		missaile.physicsBody.categoryBitMask = missileCategory;	//カテゴリー（ミサイル）
		missaile.physicsBody.contactTestBitMask = rockCategory;	//ヒットテストするオブジェクト（隕石）
		missaile.physicsBody.collisionBitMask = rockCategory;	//接触できるオブジェクト（隕石）
		
		//-------------------------
		//ミサイル発射
		//-------------------------
		CGFloat		radian = 0;
		CGFloat	x = sin(radian);
		CGFloat	y = cos(radian);
		missaile.physicsBody.velocity = CGVectorMake(-(400*x), (400*y));	//ベクトル
	}
}


-(void) showGameOver
{
	SKLabelNode *gameOverLabel = [SKLabelNode labelNodeWithFontNamed:@"Baskerville-Bold"];
	
	gameOverLabel.text = @"GAME OVER";
	gameOverLabel.fontSize = 40;
	gameOverLabel.position = CGPointMake(CGRectGetMidX(self.frame),
								   CGRectGetMidY(self.frame));
	
	[self addChild:gameOverLabel];
	
}


#pragma mark - SKPhysicsContactDelegate

- (void)didBeginContact:(SKPhysicsContact *)contact
{
	//隕石が地球に落下！
	if([contact.bodyA.node.name isEqualToString:@"earth"]||[contact.bodyB.node.name isEqualToString:@"earth"]){
		
		_gameOver = YES;
		self.physicsWorld.speed = 0;
		
		{
			NSString *path = [[NSBundle mainBundle] pathForResource:@"Smoke" ofType:@"sks"];
			SKEmitterNode*	particleBom = [NSKeyedUnarchiver unarchiveObjectWithFile:path];
			particleBom.position = contact.contactPoint;
			particleBom.numParticlesToEmit = 100;
			[self addChild:particleBom];
		}
		{
			NSString *path = [[NSBundle mainBundle] pathForResource:@"Bom" ofType:@"sks"];
			SKEmitterNode*	particleBom = [NSKeyedUnarchiver unarchiveObjectWithFile:path];
			particleBom.position = contact.contactPoint;
			particleBom.numParticlesToEmit = 350;
			[self addChild:particleBom];
		}
		
		[self showGameOver];
	}
	else{
		//パーティクル作成
		//炎
		if(_particleFire==nil){
			NSString *path = [[NSBundle mainBundle] pathForResource:@"Fire" ofType:@"sks"];
			_particleFire = [NSKeyedUnarchiver unarchiveObjectWithFile:path];
			_particleFire.position = contact.contactPoint;
			_particleFire.numParticlesToEmit = 50;
			[self addChild:_particleFire];
		}
		else{
			_particleFire.position = contact.contactPoint;
			[_particleFire resetSimulation];
		}
		
		//スパーク
		if(_particleSpark==nil){
			NSString *path = [[NSBundle mainBundle] pathForResource:@"Spark" ofType:@"sks"];
			_particleSpark = [NSKeyedUnarchiver unarchiveObjectWithFile:path];
			_particleSpark.position = contact.contactPoint;
			_particleSpark.numParticlesToEmit = 50;
			[self addChild:_particleSpark];
		}
		else{
			_particleSpark.position = contact.contactPoint;
			[_particleSpark resetSimulation];
		}
		//接触したノードを削除
		[contact.bodyA.node removeFromParent];
		[contact.bodyB.node removeFromParent];
	}
}

- (void)didEndContact:(SKPhysicsContact *)contact
{
	
}



@end
